require 'spec_helper'

describe 'RFlow::Message::Data::HTTP::Request Avro Schema' do
  let(:schema) { RFlow::Configuration.available_data_types['RFlow::Message::Data::HTTP::Request']['avro'] }

  it "should encode and decode an object" do
    request = {
      'method' => 'METHOD',
      'uri' => 'URI',
      'query_string' => 'QUERYSTRING',
      'protocol' => 'PROTOCOL',
      'headers' => {
        'header1' => 'HEADER1',
        'header2' => 'HEADER2',
      },
      'content' => '',
    }

    expect { encode_avro(schema, request) }.to_not raise_error
    encoded_request = encode_avro(schema, request)

    expect { decode_avro(schema, encoded_request) }.to_not raise_error
    decode_avro(schema, encoded_request).tap do |decoded|
      expect(decoded['method']).to eq(request['method'])
      expect(decoded['uri']).to eq(request['uri'])
      expect(decoded['query_string']).to eq(request['query_string'])
      expect(decoded['protocol']).to eq(request['protocol'])
      expect(decoded['headers']).to eq(request['headers'])
      expect(decoded['content']).to eq(request['content'])
    end
  end

  it "should encode and decode non-UTF8 data" do
    request = {
      'method' => 'METHOD',
      'uri' => "\xFF".force_encoding('binary'),
      'query_string' => "\xFF".force_encoding('binary'),
      'protocol' => 'PROTOCOL',
      'headers' => {
        'header1' => "\xFF".force_encoding('binary'),
        'header2' => "\xFF".force_encoding('binary')
      },
      'content' => '',
    }

    expect { encode_avro(schema, request) }.to_not raise_error
    encoded_request = encode_avro(schema, request)

    expect { decode_avro(schema, encoded_request) }.to_not raise_error
    decode_avro(schema, encoded_request).tap do |decoded|
      expect(decoded['method']).to eq(request['method'])
      expect(decoded['uri']).to eq(request['uri'])
      expect(decoded['query_string']).to eq(request['query_string'])
      expect(decoded['protocol']).to eq(request['protocol'])
      expect(decoded['headers']).to eq(request['headers'])
      expect(decoded['content']).to eq(request['content'])
    end
  end
end

describe 'RFlow::Message::Data::HTTP::Response Avro Schema' do
  let(:schema) { RFlow::Configuration.available_data_types['RFlow::Message::Data::HTTP::Response']['avro'] }

  it "should encode and decode an object" do
    response = {
      'protocol' => 'METHOD',
      'status_code' => 200,
      'status_reason_phrase' => 'STATUSREASONPHRASE',
      'headers' => {
        'header1' => 'HEADER1',
        'header2' => 'HEADER2',
      },
      'content' => 'CONTENT',
    }

    expect { encode_avro(schema, response) }.to_not raise_error
    encoded_response = encode_avro(schema, response)

    expect { decode_avro(schema, encoded_response) }.to_not raise_error
    decode_avro(schema, encoded_response).tap do |decoded|
      expect(decoded['protocol']).to eq(response['protocol'])
      expect(decoded['status_code']).to eq(response['status_code'])
      expect(decoded['status_reason_phrase']).to eq(response['status_reason_phrase'])
      expect(decoded['headers']).to eq(response['headers'])
      expect(decoded['content']).to eq(response['content'])
    end
  end

  it "should encode and decode non-UTF8 data" do
    response = {
      'protocol' => 'METHOD',
      'status_code' => 200,
      'status_reason_phrase' => 'STATUSREASONPHRASE',
      'headers' => {
        'header1' => "\xFF".force_encoding('binary'),
        'header2' => "\xFF".force_encoding('binary')
      },
      'content' => 'CONTENT',
    }

    expect { encode_avro(schema, response) }.to_not raise_error
    encoded_response = encode_avro(schema, response)

    expect { decode_avro(schema, encoded_response) }.to_not raise_error
    decode_avro(schema, encoded_response).tap do |decoded|
      expect(decoded['protocol']).to eq(response['protocol'])
      expect(decoded['status_code']).to eq(response['status_code'])
      expect(decoded['status_reason_phrase']).to eq(response['status_reason_phrase'])
      expect(decoded['headers']['header1']).to eq(response['headers']['header1'])
      expect(decoded['content']).to eq(response['content'])
    end
  end
end
